import DDC
import Control.Monad
import Control.Monad.State


-- an individual light state (will probably turn into a column in DDC)
-- its either powered (in which case its at full intensity) or unpowered
-- in which case its fading and the brightness is stored (0..255)
data KittLight = LightPowered | LightUnpowered Int

-- so we can store all six columns in a list

startKittLights = take 6 $ repeat (LightUnpowered 0)

-- so state in the KITT monad is [KittLight]

-- operations:
--   tick: this does nothing to the state in StateT, but renders in the
--   underlying monad (using renderKittLights on the present state)

tick :: StateT [KittLight] (State DDCMonadState) ()
tick = do
  oldstate <- get
  let newstate = map fadeKittLight oldstate
  put newstate
  lift (renderKittLights newstate oldstate)
  lift (wait 100)
  return ()

fadeKittLight LightPowered = LightPowered
fadeKittLight (LightUnpowered n) = LightUnpowered (max (n - 8) 0)

kittmain = do
  replicateM_ 5 sweep
  tickForSeconds 10

sweep = do
  sweepRight
  sweepLeft

sweepRight = 
  forM_ [0..5] $ \n -> do
    pulse n

sweepLeft = 
  forM_ (reverse [1..4]) $ \n -> do
    pulse n

pulse n = do
    lightOn n
    tickForHalfSecond
    lightOff n


-- turns on the light but does not tick
lightOn n = replaceLightState n LightPowered
lightOff n = replaceLightState n (LightUnpowered 170)


replaceLightState n newlightstate = do
  oldstate <- get
  let numberedState = oldstate `zip` [0..]
  let newstate = map (\(s,nn) -> if nn ==n then newlightstate else s) numberedState
  put newstate


tickForSeconds s = do
  replicateM_ (s * 10) tick

tickForHalfSecond = replicateM_ 5 tick

main = runDDC $ do
  runStateT kittmain startKittLights
  return ()

-- TODO only deliver a change if there is really a change
renderKittLights l o = do
  mapM_ (\n-> renderKittLight (l!!n) n) [0..5]

renderKittLight (LightPowered) col = colorColumn col red
renderKittLight (LightUnpowered intensity) col = colorColumn col (Color intensity intensity 0)
-- TODO using a different colour here to help distinguish during
-- debugging between powered and unpowered lights
  
